==> engage-android-1.0.0-sources.jar: main/tech/poool/commons/oak/Utils.kt <==
package tech.poool.commons.oak

import android.content.Context
import android.util.TypedValue
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.geometry.Size
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.TileMode
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import tech.poool.commons.oak.data.BlockClickableItem
import tech.poool.commons.oak.data.BlockColItem
import tech.poool.commons.oak.data.BlockFoldableItem
import tech.poool.commons.oak.data.BlockItem
import tech.poool.commons.oak.data.BlockRowItem
import tech.poool.commons.oak.data.BlockStyles

internal fun fromHex(hex: String): Color {
    var result = hex

    if (hex.startsWith("#") && hex.length == 4) {
        val red = hex.substring(1, 2)
        val green = hex.substring(2, 3)
        val blue = hex.substring(3, 4)
        result = "#$red$red$green$green$blue$blue"
    } else if (hex.startsWith("rgb(")) {
        val rgb = hex.substring(4, hex.length - 1).split(",")
        val red = rgb[0].trim().toInt()
        val green = rgb[1].trim().toInt()
        val blue = rgb[2].trim().toInt()
        result = String.format("#%02x%02x%02x", red, green, blue)
    } else if (hex.startsWith("rgba(")) {
        val rgba = hex.substring(5, hex.length - 1).split(",")
        val red = rgba[0].trim().toInt()
        val green = rgba[1].trim().toInt()
        val blue = rgba[2].trim().toInt()
        val alpha = rgba[3].trim().toFloat()
        result = String.format("#%02x%02x%02x%02x", red, green, blue, (alpha * 255).toInt())
    }

    return try {
        Color(android.graphics.Color.parseColor(result))
    } catch (e: Exception) {
        Color.Transparent
    }
}

/**
 * Oak sizes can be "2", "2px", "2%", and all the other css sizes (like em, rem, ...) but fuck that
 */
internal fun oakSizeToDp (
    size: String?
): Dp {
    return size?.filter { it.isDigit() }?.toIntOrNull()?.dp ?: 0.dp
}

internal fun oakToTileModeX(
    styles: BlockStyles? = null
): TileMode {
    return when (styles?.backgroundRepeat) {
        "repeat-x" -> TileMode.Repeated
        "repeat" -> TileMode.Repeated
        else -> TileMode.Decal
    }
}

internal fun oakToTileModeY(
    styles: BlockStyles? = null
): TileMode {
    return when (styles?.backgroundRepeat) {
        "repeat-y" -> TileMode.Repeated
        "repeat" -> TileMode.Repeated
        else -> TileMode.Decal
    }
}

internal fun oakToBackgroundSize(
    imageSize: Size,
    parentSize: Size,
    backgroundSize: String,
): Size {
    val mode = getAspectRatioMode(imageSize)
    val parentMode = getAspectRatioMode(parentSize)

    val aspectRatio = imageSize.width / imageSize.height
    val aspectRatioReverse = imageSize.height / imageSize.width

    val aspectRatioedWidth = Size(parentSize.height * aspectRatio, parentSize.height)
    val aspectRatioedHeight = Size(parentSize.width, parentSize.width * aspectRatioReverse)

    // We're too tired to comment, deal with it
    // TODO: Add comments
    return when (backgroundSize) {
        "cover" -> when(parentMode) {
            "square" -> when (mode) {
                "square" -> parentSize
                "landscape" -> aspectRatioedWidth
                "portrait" -> aspectRatioedHeight
                else -> imageSize
            }
            "landscape" -> when (mode) {
                "square", "portrait" -> aspectRatioedHeight
                "landscape" ->
                    if (aspectRatioedWidth.height == parentSize.height && aspectRatioedWidth.width < parentSize.width)
                        aspectRatioedHeight
                    else
                        aspectRatioedWidth
                else -> imageSize
            }
            "portrait" -> when (mode) {
                "square", "landscape" -> aspectRatioedWidth
                "portrait" ->
                    if (aspectRatioedHeight.width == parentSize.width && aspectRatioedHeight.height < parentSize.height)
                        aspectRatioedWidth
                    else
                        aspectRatioedHeight
                else -> imageSize
            }
            else -> imageSize
        }
        "contain" -> when(parentMode) {
            "square" -> when (mode) {
                "square" -> parentSize
                "landscape" -> aspectRatioedHeight
                "portrait" -> aspectRatioedWidth
                else -> imageSize
            }
            "landscape" -> when (mode) {
                "square", "portrait" -> aspectRatioedWidth
                "landscape" ->
                    if (aspectRatioedHeight.width == parentSize.width && aspectRatioedHeight.height < parentSize.height)
                        aspectRatioedHeight
                    else
                        aspectRatioedWidth
                else -> imageSize
            }
            "portrait" -> when (mode) {
                "square", "landscape" -> aspectRatioedHeight
                "portrait" ->
                    if (aspectRatioedWidth.height == parentSize.height && aspectRatioedWidth.width < parentSize.width)
                        aspectRatioedWidth
                    else
                        aspectRatioedHeight
                else -> imageSize
            }
            else -> imageSize
        }
        else -> imageSize
    }
}

internal fun getAspectRatioMode(size: Size): String {
    return if (size.width == size.height) "square"
        else if (size.width > size.height) "landscape"
        else "portrait"
}

internal fun oakToBackgroundPosition(
    imageSize: Size,
    parentSize: Size,
    position: String,
): Offset {
    return when (position) {
        "center" -> Offset(
            (parentSize.width - imageSize.width) / 2,
            (parentSize.height - imageSize.height) / 2
        )
        "left top" -> Offset(0f, 0f)
        "right top" -> Offset(parentSize.width - imageSize.width, 0f)
        "center top" -> Offset((parentSize.width - imageSize.width) / 2, 0f)
        "left center" -> Offset(0f, (parentSize.height - imageSize.height) / 2)
        "right center" -> Offset(parentSize.width - imageSize.width, (parentSize.height - imageSize.height) / 2)
        "center bottom" -> Offset((parentSize.width - imageSize.width) / 2, parentSize.height - imageSize.height)
        "left bottom" -> Offset(0f, parentSize.height - imageSize.height)
        "right bottom" -> Offset(parentSize.width - imageSize.width, parentSize.height - imageSize.height)
        else -> Offset.Zero
    }
}

/**
 * Android TextView needs px for things like line height, where compose uses sp
 */
internal fun spToPx(sp: Float, context: Context): Int {
    return TypedValue.applyDimension(
        TypedValue.COMPLEX_UNIT_SP,
        sp,
        context.resources.displayMetrics
    ).toInt()
}

internal fun getBlocksByType (
    types: List<String>,
    content: List<BlockItem>
): MutableList<BlockItem> {
    return content.fold(mutableListOf()) { acc, block ->
        if (block is BlockRowItem) {
            acc.addAll(getBlocksByType(types, block.cols))
        } else if (block is BlockColItem) {
            acc.addAll(getBlocksByType(types, block.content))
        } else if (block is BlockClickableItem) {
            acc.addAll(getBlocksByType(types, block.content))
        } else if (block is BlockFoldableItem) {
            acc.addAll(getBlocksByType(types, block.content))
            acc.addAll(getBlocksByType(types, block.seeLess))
            acc.addAll(getBlocksByType(types, block.seeMore))
        } else if (types.contains(block.type)) {
            acc.add(block)
        }

        return@fold acc
    }
}

internal fun filterBlocks (
    content: List<BlockItem>,
    discriminator: (BlockItem) -> Boolean
): MutableList<BlockItem> {
    return content.fold(mutableListOf()) { acc, block ->
        if (block is BlockRowItem) {
            acc.addAll(filterBlocks(block.cols, discriminator))
        } else if (block is BlockColItem) {
            acc.addAll(filterBlocks(block.content, discriminator))
        } else if (block is BlockClickableItem) {
            acc.addAll(filterBlocks(block.content, discriminator))
        } else if (block is BlockFoldableItem) {
            acc.addAll(filterBlocks(block.content, discriminator))
            acc.addAll(filterBlocks(block.seeLess, discriminator))
            acc.addAll(filterBlocks(block.seeMore, discriminator))
        } else if (discriminator(block)) {
            acc.add(block)
        }

        return@fold acc
    }
}

==> engage-android-1.2.1-sources.jar: main/tech/poool/commons/oak/Utils.kt <==
package tech.poool.commons.oak

import android.content.Context
import android.util.TypedValue
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.SelectableDates
import androidx.compose.runtime.Composable
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.geometry.Size
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.TileMode
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import tech.poool.commons.compose.LocalBlock
import tech.poool.commons.compose.LocalCommons
import tech.poool.commons.oak.data.BlockClickableItem
import tech.poool.commons.oak.data.BlockColItem
import tech.poool.commons.oak.data.BlockFoldableItem
import tech.poool.commons.oak.data.BlockItem
import tech.poool.commons.oak.data.BlockRowItem
import tech.poool.commons.oak.data.BlockStyles
import tech.poool.commons.oak.data.DisplayCondition
import java.text.SimpleDateFormat
import java.util.*

internal fun fromHex(hex: String): Color {
    var result = hex

    if (hex.startsWith("#") && hex.length == 4) {
        val red = hex.substring(1, 2)
        val green = hex.substring(2, 3)
        val blue = hex.substring(3, 4)
        result = "#$red$red$green$green$blue$blue"
    } else if (hex.startsWith("rgb(")) {
        val rgb = hex.substring(4, hex.length - 1).split(",")
        val red = rgb[0].trim().toInt()
        val green = rgb[1].trim().toInt()
        val blue = rgb[2].trim().toInt()
        result = String.format("#%02x%02x%02x", red, green, blue)
    } else if (hex.startsWith("rgba(")) {
        val rgba = hex.substring(5, hex.length - 1).split(",")
        val red = rgba[0].trim().toInt()
        val green = rgba[1].trim().toInt()
        val blue = rgba[2].trim().toInt()
        val alpha = rgba[3].trim().toFloat()
        result = String.format("#%02x%02x%02x%02x", red, green, blue, (alpha * 255).toInt())
    }

    return try {
        Color(android.graphics.Color.parseColor(result))
    } catch (e: Exception) {
        Color.Transparent
    }
}

internal fun millisToDate(millis: Long, formatDate: String = "dd/MM/yyyy"): String {
    val pattern = formatDate.replace("[mYD]".toRegex()) {
        if (it.value == "m") it.value.uppercase()
        else it.value.lowercase()
    }

    return SimpleDateFormat(pattern, Locale.getDefault()).format(Date(millis))
}

@OptIn(ExperimentalMaterial3Api::class)
internal fun selectableDates(
    minDate: String? = null,
    maxDate: String? = null,
): SelectableDates? {
    if (minDate.isNullOrEmpty() || maxDate.isNullOrEmpty()) return null

    val formatter = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.sss'Z'", Locale.getDefault())
    val min = formatter.parse(minDate)?.time // excluded
    val max = formatter.parse(maxDate)?.time // included

    return object : SelectableDates {
        override fun isSelectableDate(utcTimeMillis: Long): Boolean {
            if (min == null || max == null) return true

            val date = Date(utcTimeMillis)

            return date.after(Date(min)) && date.before(Date(max))
        }
    }
}

/**
 * Oak sizes can be "2", "2px", "2%", and all the other css sizes (like em, rem, ...) but fuck that
 */
internal fun oakSizeToDp (
    size: String?
): Dp {
    return size?.filter { it.isDigit() }?.toIntOrNull()?.dp ?: 0.dp
}

internal fun oakToTileModeX(
    styles: BlockStyles? = null
): TileMode {
    return when (styles?.backgroundRepeat) {
        "repeat-x" -> TileMode.Repeated
        "repeat" -> TileMode.Repeated
        else -> TileMode.Decal
    }
}

internal fun oakToTileModeY(
    styles: BlockStyles? = null
): TileMode {
    return when (styles?.backgroundRepeat) {
        "repeat-y" -> TileMode.Repeated
        "repeat" -> TileMode.Repeated
        else -> TileMode.Decal
    }
}

internal fun oakToBackgroundSize(
    imageSize: Size,
    parentSize: Size,
    backgroundSize: String,
): Size {
    val mode = getAspectRatioMode(imageSize)
    val parentMode = getAspectRatioMode(parentSize)

    val aspectRatio = imageSize.width / imageSize.height
    val aspectRatioReverse = imageSize.height / imageSize.width

    val aspectRatioedWidth = Size(parentSize.height * aspectRatio, parentSize.height)
    val aspectRatioedHeight = Size(parentSize.width, parentSize.width * aspectRatioReverse)

    // We're too tired to comment, deal with it
    // TODO: Add comments
    return when (backgroundSize) {
        "cover" -> when(parentMode) {
            "square" -> when (mode) {
                "square" -> parentSize
                "landscape" -> aspectRatioedWidth
                "portrait" -> aspectRatioedHeight
                else -> imageSize
            }
            "landscape" -> when (mode) {
                "square", "portrait" -> aspectRatioedHeight
                "landscape" ->
                    if (aspectRatioedWidth.height == parentSize.height && aspectRatioedWidth.width < parentSize.width)
                        aspectRatioedHeight
                    else
                        aspectRatioedWidth
                else -> imageSize
            }
            "portrait" -> when (mode) {
                "square", "landscape" -> aspectRatioedWidth
                "portrait" ->
                    if (aspectRatioedHeight.width == parentSize.width && aspectRatioedHeight.height < parentSize.height)
                        aspectRatioedWidth
                    else
                        aspectRatioedHeight
                else -> imageSize
            }
            else -> imageSize
        }
        "contain" -> when(parentMode) {
            "square" -> when (mode) {
                "square" -> parentSize
                "landscape" -> aspectRatioedHeight
                "portrait" -> aspectRatioedWidth
                else -> imageSize
            }
            "landscape" -> when (mode) {
                "square", "portrait" -> aspectRatioedWidth
                "landscape" ->
                    if (aspectRatioedHeight.width == parentSize.width && aspectRatioedHeight.height < parentSize.height)
                        aspectRatioedHeight
                    else
                        aspectRatioedWidth
                else -> imageSize
            }
            "portrait" -> when (mode) {
                "square", "landscape" -> aspectRatioedHeight
                "portrait" ->
                    if (aspectRatioedWidth.height == parentSize.height && aspectRatioedWidth.width < parentSize.width)
                        aspectRatioedWidth
                    else
                        aspectRatioedHeight
                else -> imageSize
            }
            else -> imageSize
        }
        else -> imageSize
    }
}

internal fun getAspectRatioMode(size: Size): String {
    return if (size.width == size.height) "square"
        else if (size.width > size.height) "landscape"
        else "portrait"
}

internal fun oakToBackgroundPosition(
    imageSize: Size,
    parentSize: Size,
    position: String,
): Offset {
    return when (position) {
        "center" -> Offset(
            (parentSize.width - imageSize.width) / 2,
            (parentSize.height - imageSize.height) / 2
        )
        "left top" -> Offset(0f, 0f)
        "right top" -> Offset(parentSize.width - imageSize.width, 0f)
        "center top" -> Offset((parentSize.width - imageSize.width) / 2, 0f)
        "left center" -> Offset(0f, (parentSize.height - imageSize.height) / 2)
        "right center" -> Offset(parentSize.width - imageSize.width, (parentSize.height - imageSize.height) / 2)
        "center bottom" -> Offset((parentSize.width - imageSize.width) / 2, parentSize.height - imageSize.height)
        "left bottom" -> Offset(0f, parentSize.height - imageSize.height)
        "right bottom" -> Offset(parentSize.width - imageSize.width, parentSize.height - imageSize.height)
        else -> Offset.Zero
    }
}

/**
 * Android TextView needs px for things like line height, where compose uses sp
 */
internal fun spToPx(sp: Float, context: Context): Int {
    return TypedValue.applyDimension(
        TypedValue.COMPLEX_UNIT_SP,
        sp,
        context.resources.displayMetrics
    ).toInt()
}

internal fun getBlocksByType (
    types: List<String>,
    content: List<BlockItem>
): MutableList<BlockItem> {
    return content.fold(mutableListOf()) { acc, block ->
        if (block is BlockRowItem) {
            acc.addAll(getBlocksByType(types, block.cols))
        } else if (block is BlockColItem) {
            acc.addAll(getBlocksByType(types, block.content))
        } else if (block is BlockClickableItem) {
            acc.addAll(getBlocksByType(types, block.content))
        } else if (block is BlockFoldableItem) {
            acc.addAll(getBlocksByType(types, block.content))
            acc.addAll(getBlocksByType(types, block.seeLess))
            acc.addAll(getBlocksByType(types, block.seeMore))
        } else if (types.contains(block.type)) {
            acc.add(block)
        }

        return@fold acc
    }
}

internal fun filterBlocks (
    content: List<BlockItem>,
    discriminator: (BlockItem) -> Boolean
): MutableList<BlockItem> {
    return content.fold(mutableListOf()) { acc, block ->
        if (block is BlockRowItem) {
            acc.addAll(filterBlocks(block.cols, discriminator))
        } else if (block is BlockColItem) {
            acc.addAll(filterBlocks(block.content, discriminator))
        } else if (block is BlockClickableItem) {
            acc.addAll(filterBlocks(block.content, discriminator))
        } else if (block is BlockFoldableItem) {
            acc.addAll(filterBlocks(block.content, discriminator))
            acc.addAll(filterBlocks(block.seeLess, discriminator))
            acc.addAll(filterBlocks(block.seeMore, discriminator))
        } else if (discriminator(block)) {
            acc.add(block)
        }

        return@fold acc
    }
}

@Composable
internal fun renderVariables(
    str: String,
    variables: Map<String, Any?>? = null,
    additional: Map<String, Any?> = emptyMap()
): String {
    val commons = LocalCommons.current
    val v = commons.variables?.getAll()
    val variables_ = variables ?: v ?: emptyMap()

    var string = str

    (additional + variables_).map { (k, v) ->
        string = string.replace("{${k}}", "" + (v.toString() ?: ""))

        return@map string
    }

    return string
}

internal fun getConditionalDisplay(
    block: BlockItem,
    context: List<String>? = null,
    group: String? = null,
    filters: List<String>? = null,
    variables: Map<String, Any?>? = null
): Boolean {
    if (block.settings?.displayConditions.isNullOrEmpty()) {
        return true
    }

    val displayConditions = block.settings?.displayConditions
        ?.filter { it.type == "show" }

    val hideConditions = block.settings?.displayConditions
        ?.filter { it.type == "hide" }

    val shouldDisplay = if (displayConditions.isNullOrEmpty()) true else displayConditions
        .all { isConditionValid(it, context, group, filters, variables) }
    val shouldHide = if (hideConditions.isNullOrEmpty()) false else hideConditions
        .all { isConditionValid(it, context, group, filters, variables) }

    return shouldDisplay && !shouldHide
}

internal fun isConditionValid(
    condition: DisplayCondition,
    context: List<String>? = null,
    group: String? = null,
    filters: List<String>? = null,
    variables: Map<String, Any?>? = null
): Boolean {
    return when (condition.target) {
        "filter" -> filters?.contains(condition.value) ?: false
        "context" -> context?.contains(condition.value) ?: false
        "group" -> group == condition.value
        "variable" -> {
            when (condition.operator) {
                "exists" -> variables?.containsKey(condition.key) ?: false
                "nexists" -> !(variables?.containsKey(condition.key) ?: false)
                "gt" -> {
                    toInt(variables?.get(condition.key)) >
                            (condition.value?.toInt() ?: 0)
                }
                "lt" -> toInt(variables?.get(condition.key)) <
                    (condition.value?.toInt() ?: 0)
                "gte" -> toInt(variables?.get(condition.key)) >=
                    (condition.value?.toInt() ?: 0)
                "lte" -> toInt(variables?.get(condition.key)) <=
                    (condition.value?.toInt() ?: 0)
                "neq" -> ("" + variables?.get(condition.key)) != ("" + condition.value)
                else -> ("" + variables?.get(condition.key)) == ("" + condition.value)
            }
        }
        else -> true
    }
}

internal fun toInt(
    value: Any?
): Int {
    return try {
        value as? Int ?: value.toString().toInt()
    } catch (e: Exception) {
        0
    }
}
